@@ -1,4 +1,4 @@
-# Copyright (c) 2013 Matt Behrens <matt@zigg.com>
+# Copyright (c) 2013, 2014 Matt Behrens <matt@zigg.com>
 #
 # Permission to use, copy, modify, and distribute this software for any
 # purpose with or without fee is hereby granted, provided that the above
@@ -25,6 +25,12 @@ from octothorpe.channel import Channel
 """AsyncAGI-supporting Asterisk Manager Interface protocol"""
 
 
+# ASCII codes for DTMF digits
+
+DTMF_ASCII_CODES = [48, 49, 50, 51, 52, 53, 54, 55,
+                    56, 57, 65, 66, 67, 68, 35, 42]
+
+
 class AGIException(Exception):
     """AGI exception"""
 
@@ -37,6 +43,17 @@ class AGIException(Exception):
         return '<AGIException code=%d message=%r>' % (self.code, self.message)
 
 
+class ResultException(Exception):
+    """Result exception (i.e. unacceptable)"""
+
+    def __init__(self, result):
+        self.result = result
+
+
+    def __repr__(self):
+        return '<ResultException result=%r>' % (self.result,)
+
+
 class UnknownCommandException(KeyError):
     """Response to unknown command received"""
 
@@ -110,7 +127,7 @@ class AsyncAGIChannel(Channel):
     def _cbAGIQueued(self, result, commandid):
         """Called when AGI is queued.
 
-        Sets up a deferred result for the eventual AGIExec event.
+        Sets up a deferred result for the eventual AsyncAGI Exec event.
 
         """
         d = self.pendingAGI[commandid] = Deferred()
@@ -120,7 +137,7 @@ class AsyncAGIChannel(Channel):
     def sendAGI(self, command):
         """Queue an AGI command.
 
-        Returns a Deferred that will fire when a Success AGIExec event
+        Returns a Deferred that will fire when the AsyncAGI Exec event
         is received.
 
         """
@@ -134,6 +151,47 @@ class AsyncAGIChannel(Channel):
         return d
 
 
+    def _checkResult(self, result, acceptable):
+        """Check the result and raise a ResultException if unacceptable.
+
+        Returns just the result code to the next callback.
+
+        """
+        agiResult = result[0]
+        if agiResult in acceptable:
+            return agiResult
+        raise ResultException(agiResult)
+
+
+    def sendAGIExecPlayback(self, filename, background=False):
+        """Queue an AGI EXEC of the Playback or Background application.
+
+        Returns a Deferred that will fire with the AGI result code (e.g.
+        0, or the ASCII code for the interrupting digit in background
+        mode) when the application has executed.
+
+        """
+        if background:
+            d = self.sendAGI('EXEC Background %s' % (filename,))
+            d.addCallback(self._checkResult, [0] + DTMF_ASCII_CODES)
+        else:
+            d = self.sendAGI('EXEC Playback %s' % (filename,))
+            d.addCallback(self._checkResult, [0])
+        return d
+
+
+    def sendAGIHangup(self):
+        """Queue an AGI HANGUP.
+
+        Returns a Deferred that will fire when the hangup has
+        succeeded.
+
+        """
+        d = self.sendAGI('HANGUP')
+        d.addCallback(self._checkResult, [1])
+        return d
+
+
 class AsyncAGIProtocol(AMIProtocol):
     """AsyncAGI-supporting AMI protocol"""
 
