@@ -1,4 +1,4 @@
-# Copyright (c) 2013 Matt Behrens <matt@zigg.com>
+# Copyright (c) 2013, 2014 Matt Behrens <matt@zigg.com>
 #
 # Permission to use, copy, modify, and distribute this software for any
 # purpose with or without fee is hereby granted, provided that the above
@@ -20,7 +20,7 @@ from twisted.trial import unittest
 from twisted.test import proto_helpers
 
 from octothorpe.asyncagi import AGIException, AsyncAGIProtocol, AsyncAGIChannel
-from octothorpe.asyncagi import UnknownCommandException
+from octothorpe.asyncagi import ResultException, UnknownCommandException
 from octothorpe.base import ProtocolError
 from octothorpe.test.test_base import disassembleMessage
 
@@ -62,6 +62,15 @@ class AsyncAGIProtocolTestCase(unittest.TestCase):
         )
 
 
+    def test_ResultExceptionRepr(self):
+        """repr() of a PlaybackException"""
+
+        self.assertEqual(
+            repr(ResultException(64)),
+            '<ResultException result=64>'
+        )
+
+
     def test_asyncAGIStart(self):
         """Respond to an AsyncAGI Start event"""
 
@@ -288,4 +297,174 @@ class AsyncAGIProtocolTestCase(unittest.TestCase):
         self.assertEqual(len(self.flushLoggedErrors(ProtocolError)), 0)
 
 
+    def test_AGIExecPlayback(self):
+        """Successfully execute the Playback command"""
+
+        channel = self._spawnChannel()
+        d = channel.sendAGIExecPlayback('hello-world')
+
+        message = disassembleMessage(self.transport.value())
+        self.assertEqual(message['action'], 'AGI')
+        self.assertIn('actionid', message)
+        self.assertEqual(message['command'], 'EXEC Playback hello-world')
+        self.assertIn('commandid', message)
+
+        cbSuccess = Mock()
+        d.addCallback(cbSuccess)
+
+        self.protocol.dataReceived(
+            'Response: Success\r\n'
+            'ActionID: ' + message['actionid'] + '\r\n'
+            '\r\n'
+        )
+        self.assertEqual(len(cbSuccess.mock_calls), 0)
+
+        self.protocol.dataReceived(
+            'Event: AsyncAGI\r\n'
+            'SubEvent: Exec\r\n'
+            'Channel: Foo/202-0\r\n'
+            'CommandID: ' + message['commandid'] + '\r\n'
+            'Result: ' + quote('200 result=0\n') + '\r\n'
+            '\r\n'
+        )
+        cbSuccess.assert_called_once_with(0)
+
+
+    def test_AGIExecPlaybackFailed(self):
+        """Fail to execute the Playback command"""
+
+        channel = self._spawnChannel()
+        d = channel.sendAGIExecPlayback('hello-world')
+
+        message = disassembleMessage(self.transport.value())
+        self.assertEqual(message['action'], 'AGI')
+        self.assertIn('actionid', message)
+        self.assertEqual(message['command'], 'EXEC Playback hello-world')
+        self.assertIn('commandid', message)
+
+        playbackFailed = Mock()
+        d.addErrback(playbackFailed)
+
+        self.protocol.dataReceived(
+            'Response: Success\r\n'
+            'ActionID: ' + message['actionid'] + '\r\n'
+            '\r\n'
+        )
+        self.assertEqual(len(playbackFailed.mock_calls), 0)
+
+        self.protocol.dataReceived(
+            'Event: AsyncAGI\r\n'
+            'SubEvent: Exec\r\n'
+            'Channel: Foo/202-0\r\n'
+            'CommandID: ' + message['commandid'] + '\r\n'
+            'Result: '
+            '' + quote('200 result=-1\n') + ''
+            '\r\n'
+            '\r\n'
+        )
+
+        self.assertEqual(len(playbackFailed.mock_calls), 1)
+
+
+    def test_AGIExecPlaybackBackground(self):
+        """Successfully execute Playback in background mode"""
+
+        channel = self._spawnChannel()
+        d = channel.sendAGIExecPlayback('hello-world', background=True)
+
+        message = disassembleMessage(self.transport.value())
+        self.assertEqual(message['action'], 'AGI')
+        self.assertIn('actionid', message)
+        self.assertEqual(message['command'], 'EXEC Background hello-world')
+        self.assertIn('commandid', message)
+
+        cbSuccess = Mock()
+        d.addCallback(cbSuccess)
+
+        self.protocol.dataReceived(
+            'Response: Success\r\n'
+            'ActionID: ' + message['actionid'] + '\r\n'
+            '\r\n'
+        )
+        self.assertEqual(len(cbSuccess.mock_calls), 0)
+
+        self.protocol.dataReceived(
+            'Event: AsyncAGI\r\n'
+            'SubEvent: Exec\r\n'
+            'Channel: Foo/202-0\r\n'
+            'CommandID: ' + message['commandid'] + '\r\n'
+            'Result: ' + quote('200 result=48\n') + '\r\n'
+            '\r\n'
+        )
+        cbSuccess.assert_called_once_with(48)
+
+
+    def test_AGIHangup(self):
+        """Successfully send an AGI hangup"""
+
+        channel = self._spawnChannel()
+        d = channel.sendAGIHangup()
+
+        message = disassembleMessage(self.transport.value())
+        self.assertEqual(message['action'], 'AGI')
+        self.assertIn('actionid', message)
+        self.assertEqual(message['command'], 'HANGUP')
+        self.assertIn('commandid', message)
+
+        cbSuccess = Mock()
+        d.addCallback(cbSuccess)
+
+        self.protocol.dataReceived(
+            'Response: Success\r\n'
+            'ActionID: ' + message['actionid'] + '\r\n'
+            '\r\n'
+        )
+        self.assertEqual(len(cbSuccess.mock_calls), 0)
+
+        self.protocol.dataReceived(
+            'Event: AsyncAGI\r\n'
+            'SubEvent: Exec\r\n'
+            'Channel: Foo/202-0\r\n'
+            'CommandID: ' + message['commandid'] + '\r\n'
+            'Result: ' + quote('200 result=1\n') + '\r\n'
+            '\r\n'
+        )
+        cbSuccess.assert_called_once_with(1)
+
+
+    def test_AGIHangupFailed(self):
+        """Fail to hang up the channel"""
+
+        channel = self._spawnChannel()
+        d = channel.sendAGIHangup()
+
+        message = disassembleMessage(self.transport.value())
+        self.assertEqual(message['action'], 'AGI')
+        self.assertIn('actionid', message)
+        self.assertEqual(message['command'], 'HANGUP')
+        self.assertIn('commandid', message)
+
+        hangupFailed = Mock()
+        d.addErrback(hangupFailed)
+
+        self.protocol.dataReceived(
+            'Response: Success\r\n'
+            'ActionID: ' + message['actionid'] + '\r\n'
+            '\r\n'
+        )
+        self.assertEqual(len(hangupFailed.mock_calls), 0)
+
+        self.protocol.dataReceived(
+            'Event: AsyncAGI\r\n'
+            'SubEvent: Exec\r\n'
+            'Channel: Foo/202-0\r\n'
+            'CommandID: ' + message['commandid'] + '\r\n'
+            'Result: '
+            '' + quote('200 result=0\n') + ''
+            '\r\n'
+            '\r\n'
+        )
+        self.assertEqual(len(hangupFailed.mock_calls), 1)
+
+
 # vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4
